//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//---------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;

using DesignSurfaceExtenderLibrary.Extensions;

namespace DesignSurfaceExtenderLibrary
{
    /// <summary>
    /// The <see cref="DesignSurfaceExtenderActionList"/> class provides the
    /// basic <see cref="DesignerActionItem"/> entries for the <see cref="DesignSurfaceExtender"/>
    /// component.
    /// </summary>
    /// <remarks>By default, the <see cref="DesignSurfaceExtender"/> component has
    /// two <see cref="DesignerActionMethodItem"/> entries:
    /// <list type="bullet">
    /// <item>Enable All Extensions</item>
    /// <item>Disable All Extensions</item>
    /// </list>
    /// The <see cref="DesignSurfaceExtenderDesigner"/> populates the action list
    /// collection with action lists from all the available extensions. 
    /// </remarks>
    internal class DesignSurfaceExtenderActionList : DesignerActionList
    {
        private ReadOnlyCollection<DesignSurfaceExtension> extensionsValue;

        private readonly string enableAllPropertyName = "EnableAll";
        private readonly string enableAllDisplayText = "Enable All Extensions";

        private readonly string disableAllPropertyName = "DisableAll";
        private readonly string disableAllDisplayText = "Disable All Extensions";

        // This field backs the UiService property.
        private DesignerActionUIService uiService;

        /// <summary>
        /// The <see cref="DesignSurfaceExtenderActionList"/> constructor has
        /// private access, to enforce the factory pattern. 
        /// </summary>
        /// <param name="component">The component attached to this action list.</param>
        private DesignSurfaceExtenderActionList(IComponent component )
            : base( component )
        {
        }

        public static DesignSurfaceExtenderActionList Create(
            IComponent component,
            ReadOnlyCollection<DesignSurfaceExtension> extensions )
        {
            DesignSurfaceExtenderActionList actionList = new DesignSurfaceExtenderActionList( component );
            actionList.Extensions = extensions;

            return actionList;
        }

        public override DesignerActionItemCollection GetSortedActionItems()
        {
            DesignerActionItemCollection items =
                new DesignerActionItemCollection();

            items.Add( new DesignerActionMethodItem(
                this,
                enableAllPropertyName,
                enableAllDisplayText ) );

            items.Add( new DesignerActionMethodItem(
                this,
                disableAllPropertyName,
                disableAllDisplayText ) );

            return items;
        }

        public ReadOnlyCollection<DesignSurfaceExtension> Extensions
        {
            get
            {
                return this.extensionsValue;
            }

            set
            {
                this.extensionsValue = value;
            }
        }

        internal DesignerActionUIService UiService
        {
            get
            {
                if( this.uiService == null )
                {
                    this.uiService = GetService( typeof( DesignerActionUIService ) ) as DesignerActionUIService;
                }

                return uiService;
            }
        }

        internal void EnableAll()
        {
            for( int i = 0; i < this.extensionsValue.Count; i++ )
            {
                this.extensionsValue[i].Enabled = true;
            }
             
            this.UiService.Refresh( this.Component );
        }

        internal void DisableAll()
        {
            for( int i = 0; i < this.extensionsValue.Count; i++ )
            {
                this.extensionsValue[i].Enabled = false;
            }

            this.UiService.Refresh( this.Component );
        }
    }
}
