'---------------------------------------------------------------------
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
'KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'PARTICULAR PURPOSE.
'---------------------------------------------------------------------

Option Explicit On
Option Strict On

Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Drawing
Imports System.Drawing.Design
Imports System.Windows.Forms
Imports System.Windows.Forms.Design
Imports System.Windows.Forms.Design.Behavior

Namespace Extensions

    Friend Class AnchorGlyph
        Inherits ExtensionGlyph

        ' This defines the bounds used for hit testing.
        ' These bounds are typically different than the bounds 
        ' of the glyph itself.
        Protected hitBoundsValue As Rectangle

        ' This is the cursor returned if hit test is positive.
        Protected hitTestCursor As Cursor = Cursors.Hand

        ' This defines the AnchorStyle which this glyph represents.
        Private anchorStyleValue As AnchorStyles

        ' This defines the size of the anchor glyphs.
        Private Const glyphSize As Integer = 6

        ' This defines the size of the hit bounds for an AnchorGlyph.
        Private Const hitBoundSize As Integer = glyphSize + 4

        Friend Sub New( _
        ByVal c As Control, _
        ByVal anchorStyle As AnchorStyles, _
        ByVal behaviorService As BehaviorService, _
        ByVal changeService As IComponentChangeService, _
        ByVal selectionService As ISelectionService, _
        ByVal referenceService As IReferenceService, _
        ByVal serviceProvider As IServiceProvider, _
        ByVal adorner As Adorner)
            MyBase.New( _
                c, _
                behaviorService, _
                changeService, _
                selectionService, _
                referenceService, _
                serviceProvider, _
                New AnchorBehavior(), _
                adorner)
            Me.anchorStyleValue = anchorStyle

            ' Compute the bounds of this glyph.
            Me.ComputeGeometry()
        End Sub

        Friend ReadOnly Property AnchorStyle() As AnchorStyles
            Get
                Return Me.anchorStyleValue
            End Get
        End Property

#Region "Overrides"

        ' This method renders the AnchorGlyph as a filled rectangle
        ' if the glyph is enabled, or as an open rectangle if the
        ' glyph is disabled.
        Public Overrides Sub Paint(ByVal e As PaintEventArgs)
            If Me.IsEnabled Then
                Using b As Brush = New SolidBrush(Color.Tomato)
                    e.Graphics.FillRectangle(b, Me.Bounds)
                End Using
            Else
                Using p As Pen = New Pen(Color.Tomato)
                    e.Graphics.DrawRectangle(p, Me.Bounds)
                End Using
            End If
        End Sub

        ' An AnchorGlyph has keyboard and mouse interaction, so it's
        ' important to return a cursor when the mouse is located in 
        ' the glyph's hit region. When this occurs, the 
        ' AnchorBehavior becomes active.

        Public Overrides Function GetHitTest(ByVal p As Point) As Cursor
            If hitBoundsValue.Contains(p) Then
                Return hitTestCursor
            End If

            Return Nothing
        End Function

#End Region

        Protected Overrides Function PopulateDependentProperties() As ReadOnlyCollection(Of String)
            Dim properties As List(Of String) = New List(Of String)

            properties.Add("Anchor")
            properties.Add("Size")
            properties.Add("Height")
            properties.Add("Width")
            properties.Add("Location")

            Return New ReadOnlyCollection(Of String)(properties)
        End Function

#Region "Implementation"

        ' This utility method computes the position and size of 
        ' the AnchorGlyph in the Adorner window's coordinates.
        ' It also computes the hit test bounds, which are
        ' slightly larger than the glyph's bounds.
        Protected Overrides Sub ComputeGeometry()
            Dim translatedBounds As Rectangle = New Rectangle( _
                Me.behaviorService.ControlToAdornerWindow(MyBase.RelatedControl), _
                MyBase.RelatedControl.Size)

            If (Me.anchorStyleValue And AnchorStyles.Top) = AnchorStyles.Top Then
                Me.boundsValue = New Rectangle( _
                    translatedBounds.X + CInt((translatedBounds.Width / 2) - (glyphSize / 2)), _
                    translatedBounds.Y + glyphSize, _
                    glyphSize, _
                    glyphSize)
            End If
            If (Me.anchorStyleValue And AnchorStyles.Bottom) = AnchorStyles.Bottom Then
                Me.boundsValue = New Rectangle( _
                    translatedBounds.X + CInt((translatedBounds.Width / 2) - (glyphSize / 2)), _
                    translatedBounds.Bottom - 2 * glyphSize, _
                    glyphSize, _
                    glyphSize)
            End If
            If (Me.anchorStyleValue And AnchorStyles.Left) = AnchorStyles.Left Then
                Me.boundsValue = New Rectangle( _
                    translatedBounds.X + glyphSize, _
                    translatedBounds.Y + CInt((translatedBounds.Height / 2) - (glyphSize / 2)), _
                    glyphSize, _
                    glyphSize)
            End If
            If (Me.anchorStyleValue And AnchorStyles.Right) = AnchorStyles.Right Then
                Me.boundsValue = New Rectangle( _
                    translatedBounds.Right - 2 * glyphSize, _
                    translatedBounds.Y + CInt((translatedBounds.Height / 2) - (glyphSize / 2)), _
                    glyphSize, _
                    glyphSize)
            End If

            Me.hitBoundsValue = New Rectangle( _
                Me.boundsValue.Left - CInt(hitBoundSize / 2), _
                Me.boundsValue.Top - CInt(hitBoundSize / 2), _
                hitBoundSize, _
                hitBoundSize)
        End Sub

        ' This utility property determines if the AnchorGlyph is 
        ' enabled, according to the value specified by the 
        ' control's Anchor property.
        Friend ReadOnly Property IsEnabled() As Boolean
            Get
                Return ((Me.anchorStyleValue And MyBase.RelatedControl.Anchor) = Me.anchorStyleValue)
            End Get
        End Property

#End Region

    End Class

End Namespace
