'---------------------------------------------------------------------
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
'KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'PARTICULAR PURPOSE.
'---------------------------------------------------------------------

Option Explicit On
Option Strict On

Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Diagnostics
Imports System.Drawing
Imports System.Drawing.Design
Imports System.Windows.Forms
Imports System.Windows.Forms.Design
Imports System.Windows.Forms.Design.Behavior

Namespace Extensions

    ''' <summary>
    ''' Provides an abstract base implementation for <see cref="Glyph"/> types
    ''' which represent the bounds of a control.  
    ''' </summary>
    ''' <remarks>This implementation assumes at least one <see cref="BoundsGlyph"/> is
    ''' assigned on a per-control basis.
    ''' </remarks>
    Friend MustInherit Class BoundsGlyph
        Inherits ExtensionGlyph

        Friend ReadOnly Property HitTestWidth() As Integer
            Get
                Return 4
            End Get
        End Property

        Protected boundsLeftRect As Rectangle
        Protected boundsTopRect As Rectangle
        Protected boundsRightRect As Rectangle
        Protected boundsBottomRect As Rectangle

        Protected propertyName As String

        Friend Sub New( _
        ByVal c As Control, _
        ByVal behaviorService As BehaviorService, _
        ByVal changeService As IComponentChangeService, _
        ByVal selectionService As ISelectionService, _
        ByVal referenceService As IReferenceService, _
        ByVal serviceProvider As IServiceProvider, _
        ByVal boundsBehavior As BoundsBehavior, _
        ByVal adorner As Adorner)

            MyBase.New( _
                c, _
                behaviorService, _
                changeService, _
                selectionService, _
                referenceService, _
                serviceProvider, _
                boundsBehavior, _
                adorner)

        End Sub

        Protected Overrides Function PopulateDependentProperties() As ReadOnlyCollection(Of String)
            Dim properties As List(Of String) = New List(Of String)

            properties.Add("Margin")
            properties.Add("Padding")
            properties.Add("Size")
            properties.Add("Height")
            properties.Add("Width")
            properties.Add("Location")

            Return New ReadOnlyCollection(Of String)(properties)
        End Function

        Public Overrides Function GetHitTest(ByVal p As Point) As Cursor
            Dim c As Cursor = Nothing

            If Me.boundsLeftRect.Contains(p) Or _
                Me.boundsRightRect.Contains(p) Then
                c = Cursors.SizeWE
            ElseIf Me.boundsTopRect.Contains(p) Or _
                Me.boundsBottomRect.Contains(p) Then
                c = Cursors.SizeNS
            End If

            Return c
        End Function

        ' This method renders the glyph as a simple focus rectangle.
        Public Overrides Sub Paint(ByVal e As PaintEventArgs)
            ControlPaint.DrawFocusRectangle( _
                    e.Graphics, _
                    Me.Bounds)
        End Sub

        Friend Function SelectedEdge(ByVal loc As Point) As EdgeSelection
            Dim _selectedEdge As EdgeSelection = EdgeSelection.None

            If Me.GetHitTest(loc) IsNot Nothing Then
                If Me.boundsLeftRect.Contains(loc) Then
                    _selectedEdge = EdgeSelection.Left
                ElseIf Me.boundsTopRect.Contains(loc) Then
                    _selectedEdge = EdgeSelection.Top
                ElseIf Me.boundsRightRect.Contains(loc) Then
                    _selectedEdge = EdgeSelection.Right
                ElseIf Me.boundsBottomRect.Contains(loc) Then
                    _selectedEdge = EdgeSelection.Bottom
                End If
            End If

            Return _selectedEdge
        End Function

    End Class

End Namespace

